@@ -30,6 +30,7 @@ DastStateDict = {
 
 ANDROID_EXTENSIONS = ['.apk', '.apks', '.zip', '.aab']
 
-TRY_COUNT = 360
+TRY = 360
+LONG_TRY = 20160
 END_SCAN_TIMEOUT = 30
 SLEEP_TIMEOUT = 10
