@@ -1,22 +1,23 @@
 import argparse
 import json
 import logging
+import os
 import sys
 import time
-import os
 
 import urllib3
+
 from mdast_cli.distribution_systems.app_center import AppCenter
+from mdast_cli.distribution_systems.appgallery import appgallery_download_app
 from mdast_cli.distribution_systems.appstore import AppStore
 from mdast_cli.distribution_systems.firebase import Firebase
 from mdast_cli.distribution_systems.google_play import GooglePlay
 from mdast_cli.distribution_systems.nexus import NexusRepository
 from mdast_cli.distribution_systems.nexus2 import Nexus2Repository
 from mdast_cli.distribution_systems.rustore import rustore_download_app
-from mdast_cli.helpers.const import END_SCAN_TIMEOUT, SLEEP_TIMEOUT, TRY_COUNT, DastState, DastStateDict, \
-    ANDROID_EXTENSIONS
+from mdast_cli.helpers.const import (ANDROID_EXTENSIONS, END_SCAN_TIMEOUT, LONG_TRY, SLEEP_TIMEOUT, TRY, DastState,
+                                     DastStateDict)
 from mdast_cli.helpers.helpers import check_app_md5
-
 from mdast_cli_core.token import mDastToken as mDast
 
 logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s %(message)s', datefmt='%d/%m/%Y %H:%M:%S')
@@ -33,7 +34,7 @@ def parse_args():
                                                                        'file/appcenter/nexus'
                                                                        '/firebase/appstore/google_play/rustore',
                         choices=['file', 'appcenter', 'nexus', 'nexus2', 'firebase', 'appstore', 'google_play',
-                                 'rustore'],
+                                 'rustore', 'appgallery'],
                         required=True)
 
     # Arguments used for distribution_system = file
@@ -128,7 +129,13 @@ def parse_args():
                              'if distribution system set to "appstore"')
     parser.add_argument('--appstore_password2FA', type=str,
                         help='Your password and 2FA code for iTunes login, format: password2FA_code (password1337)'
-                             'This argument is required if distribution system set to "appstore"')
+                             'DEPRECATED, will be deleted on 01.05.2023')
+    parser.add_argument('--appstore_password', type=str,
+                        help='Your password for iTunes login. This argument is required '
+                             'if distribution system set to "appstore"')
+    parser.add_argument('--appstore_2FA', type=str, help='Your 2FA code for iTunes login(6 numbers). '
+                                                         'This argument is required '
+                                                         'if distribution system set to "appstore"')
     parser.add_argument('--appstore_file_name', type=str,
                         help='File name for downloaded application.'
                              ' This argument is optional if distribution system set to "appstore"')
@@ -164,6 +171,15 @@ def parse_args():
                         help='Application package name from Rustore. This argument is required if '
                              'distribution system set to "rustore"')
 
+    # Arguments for Appgallery
+    parser.add_argument('--appgallery_app_id', type=str,
+                        help='Application id from Appgallery. This argument is required if '
+                             'distribution system set to "appgallery".'
+                             ' You can get it on app page from url, format: .../id{APP_ID}. example: C13371337')
+    parser.add_argument('--appgallery_file_name', type=str,
+                        help='File name for downloaded application.'
+                             ' This argument is optional if distribution system set to "appgallery"')
+
     # Arguments for Nexus2
     parser.add_argument('--nexus2_url', type=str,
                         help='Http url for Nexus server where mobile application is located.'
@@ -209,6 +225,7 @@ def parse_args():
                              'and exit')
     parser.add_argument('--download_path', '-p', type=str, help='Path to folder with downloaded apps',
                         default='downloaded_apps')
+    parser.add_argument('--long_wait', action='store_true', help='Time limit - 1 week for scan finish')
 
     args = parser.parse_args()
 
@@ -267,9 +284,10 @@ def parse_args():
     elif args.distribution_system == 'appstore' and (
             (args.appstore_app_id is None and args.appstore_bundle_id is None) or
             args.appstore_apple_id is None or
-            args.appstore_password2FA is None):
+            (args.appstore_password is None or args.appstore_2FA is None) and args.appstore_password2FA is None):
         parser.error('"--distribution_system appstore" requires either "--appstore_app_id" or "--appstore_bundle_id", '
-                     '"--appstore_apple_id", "--appstore_password2FA" arguments to be set')
+                     '"--appstore_apple_id" and ("--appstore_password" + "--appstore_2FA")/'
+                     '(deprecated "--appstore_password2FA") arguments to be set')
 
     elif args.distribution_system == 'google_play' and (
             args.google_play_package_name is None or
@@ -282,6 +300,9 @@ def parse_args():
     elif args.distribution_system == 'rustore' and args.rustore_package_name is None:
         parser.error('"--distribution_system rustore" requires "--rustore_package_name" to be set')
 
+    elif args.distribution_system == 'appgallery' and args.appgallery_app_id is None:
+        parser.error('"--distribution_system appgallery" requires "--appgallery_app_id" to be set')
+
     return args
 
 
@@ -303,6 +324,7 @@ def main():
         json_summary_file_name = arguments.summary_report_json_file_name
         pdf_report_file_name = arguments.pdf_report_file_name
         not_wait_scan_end = arguments.nowait
+        long_wait = arguments.long_wait
 
         url = url if url.endswith('/') else f'{url}/'
         url = url if url.endswith('rest/') else f'{url}rest'
@@ -358,8 +380,12 @@ def main():
                                              arguments.firebase_file_name)
 
         elif distribution_system == 'appstore':
+            if arguments.appstore_password and arguments.appstore_2FA:
+                password2FA = arguments.appstore_password + arguments.appstore_2FA
+            else:
+                password2FA = arguments.appstore_password2FA
             appstore = AppStore(arguments.appstore_apple_id,
-                                arguments.appstore_password2FA)
+                                password2FA)
             app_file, appstore_app_md5 = appstore.download_app(download_path,
                                                                arguments.appstore_app_id,
                                                                arguments.appstore_bundle_id,
@@ -383,6 +409,11 @@ def main():
             package_name = arguments.rustore_package_name
             app_file = rustore_download_app(package_name, download_path)
 
+        elif distribution_system == 'appgallery':
+            app_file = appgallery_download_app(arguments.appgallery_app_id,
+                                               download_path,
+                                               arguments.appgallery_file_name)
+
     except Exception as e:
         logger.fatal(f'Cannot download application file: {e}')
         exit(4)
@@ -521,7 +552,12 @@ def main():
     logger.info(f"Current scan status: {DastStateDict.get(dast_status)}")
     count = 0
 
-    while dast_status in (DastState.CREATED, DastState.INITIALIZING, DastState.STARTING) and count < TRY_COUNT:
+    if long_wait:
+        try_count = LONG_TRY
+    else:
+        try_count = TRY
+
+    while dast_status in (DastState.CREATED, DastState.INITIALIZING, DastState.STARTING) and count < try_count:
         logger.info(f"Try to get scan status for scan id {dast['id']}. Count number {count}")
         get_dast_info_resp = mdast.get_scan_info(dast['id'])
         if get_dast_info_resp.status_code != 200:
@@ -549,7 +585,7 @@ def main():
         sys.exit(1)
     count = 0
 
-    while dast_status in (DastState.STARTED, DastState.STOPPING, DastState.ANALYZING) and count < TRY_COUNT:
+    while dast_status in (DastState.STARTED, DastState.STOPPING, DastState.ANALYZING) and count < try_count:
         if count == 0 and scan_type == 'manual':
             if dast_status not in (DastState.ANALYZING, DastState.SUCCESS):
                 logger.info(f"This is manual scan with dynamic modules,"
